@@ -6,6 +6,19 @@ fn parse_canonical_path(path: &str) -> Result<String, std::io::Error> {
     Ok(std::fs::canonicalize(path)?.to_string_lossy().to_string())
 }
 
+fn parse_ensured_directory(path: &str) -> Result<String, Box<dyn std::error::Error>> {
+    std::fs::create_dir_all(&path)?;
+    Ok(std::fs::canonicalize(path)?.to_string_lossy().to_string())
+}
+
+fn parse_env_var(value: &str) -> Result<(String, String), Box<dyn std::error::Error>> {
+    let parts: Vec<&str> = value.splitn(2, '=').collect();
+    if parts.len() != 2 {
+        return Err(format!("Invalid KEY=value formatting in '{}'", value).into());
+    }
+    Ok((parts[0].to_string(), parts[1].to_string()))
+}
+
 #[derive(structopt::StructOpt, Clone, Debug, PartialEq)]
 struct CommonOpts {
     /// Exit codes that should be considered successful (comma-separated) [default: 0]
@@ -46,13 +59,25 @@ struct CommonOpts {
     #[structopt(long)]
     no_log_cmd: bool,
 
-    /// Command to run as a service
-    #[structopt(required(true), last(true))]
-    command: Vec<String>,
+    /// Write log file to a custom directory. This directory will be created if it doesn't exist.
+    #[structopt(long, value_name = "path", parse(try_from_str = parse_ensured_directory))]
+    log_dir: Option<String>,
 
     /// Append the service start arguments to the command
     #[structopt(long)]
     pass_start_args: bool,
+
+    /// Additional environment variable in the format 'KEY=value' (repeatable)
+    #[structopt(long, number_of_values = 1, parse(try_from_str = parse_env_var))]
+    env: Vec<(String, String)>,
+
+    /// Additional directory to add to the PATH environment variable (repeatable)
+    #[structopt(long, number_of_values = 1, parse(try_from_str = parse_canonical_path))]
+    path: Vec<String>,
+
+    /// Command to run as a service
+    #[structopt(required(true), last(true))]
+    command: Vec<String>,
 }
 
 #[derive(structopt::StructOpt, Clone, Debug, PartialEq)]
@@ -100,7 +125,11 @@ struct Cli {
     sub: Subcommand,
 }
 
-fn prepare_logging(name: &str, console: bool) -> Result<(), Box<dyn std::error::Error>> {
+fn prepare_logging(
+    name: &str,
+    log_dir: Option<String>,
+    console: bool,
+) -> Result<(), Box<dyn std::error::Error>> {
     let mut exe_dir = std::env::current_exe()?;
     exe_dir.pop();
 
@@ -125,6 +154,11 @@ fn prepare_logging(name: &str, console: bool) -> Result<(), Box<dyn std::error::
         })
         .format_for_stderr(|w, _now, record| write!(w, "[{}] {}", record.level(), &record.args()));
 
+    // Set custom log directory
+    if let Some(dir) = log_dir {
+        logger = logger.o_directory(Some(dir));
+    }
+
     if console {
         logger = logger.duplicate_to_stderr(flexi_logger::Duplicate::Info);
     }
@@ -230,7 +264,7 @@ fn construct_shawl_run_args(name: &str, cwd: &Option<String>, opts: &CommonOpts)
     }
     if let Some(cwd) = &cwd {
         shawl_args.push("--cwd".to_string());
-        shawl_args.push(quote(&cwd));
+        shawl_args.push(quote(cwd));
     };
     if opts.no_log {
         shawl_args.push("--no-log".to_string());
@@ -238,9 +272,25 @@ fn construct_shawl_run_args(name: &str, cwd: &Option<String>, opts: &CommonOpts)
     if opts.no_log_cmd {
         shawl_args.push("--no-log-cmd".to_string());
     }
+    if let Some(log_dir) = &opts.log_dir {
+        shawl_args.push("--log-dir".to_string());
+        shawl_args.push(quote(log_dir));
+    }
     if opts.pass_start_args {
         shawl_args.push("--pass-start-args".to_string());
     }
+    if !opts.env.is_empty() {
+        for (x, y) in &opts.env {
+            shawl_args.push("--env".to_string());
+            shawl_args.push(quote(&format!("{}={}", x, y)));
+        }
+    }
+    if !opts.path.is_empty() {
+        for path in &opts.path {
+            shawl_args.push("--path".to_string());
+            shawl_args.push(quote(path));
+        }
+    }
     shawl_args
 }
 
@@ -270,7 +320,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             Subcommand::Add { name, .. } => name,
             Subcommand::Run { name, .. } => name,
         };
-        prepare_logging(&name, console)?;
+        let log_dir = match cli.clone().sub {
+            Subcommand::Add { common, .. } => common.log_dir,
+            Subcommand::Run { common, .. } => common.log_dir,
+        };
+        prepare_logging(&name, log_dir, console)?;
     }
 
     debug!("********** LAUNCH **********");
@@ -448,8 +502,27 @@ mod service {
                 } else {
                     std::process::Stdio::null()
                 });
+            for (key, value) in &opts.env {
+                child_cmd.env(key, value);
+            }
+            if !opts.path.is_empty() {
+                child_cmd.env(
+                    "PATH",
+                    match std::env::var("PATH") {
+                        Ok(path) => format!("{};{}", path, &opts.path.join(";")),
+                        Err(_) => opts.path.join(";").to_string(),
+                    },
+                );
+            }
             if let Some(active_cwd) = &cwd {
                 child_cmd.current_dir(active_cwd);
+                child_cmd.env(
+                    "PATH",
+                    match std::env::var("PATH") {
+                        Ok(path) => format!("{};{}", path, active_cwd),
+                        Err(_) => active_cwd.to_string(),
+                    },
+                );
             }
             let mut child = match child_cmd.spawn() {
                 Ok(c) => c,
@@ -648,6 +721,10 @@ speculate::speculate! {
         text.to_string()
     }
 
+    fn p(path: &str) -> String {
+        std::fs::canonicalize(&path).unwrap().to_string_lossy().to_string()
+    }
+
     describe "CLI" {
         describe "run subcommand" {
             it "works with minimal arguments" {
@@ -666,8 +743,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -697,8 +777,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -721,8 +804,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -745,8 +831,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -769,8 +858,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -793,8 +885,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -817,8 +912,11 @@ speculate::speculate! {
                                 stop_timeout: Some(500),
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -841,8 +939,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -867,8 +968,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -905,8 +1009,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -929,8 +1036,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -953,8 +1063,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -977,8 +1090,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1001,8 +1117,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1025,8 +1144,11 @@ speculate::speculate! {
                                 stop_timeout: Some(500),
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1049,8 +1171,11 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: true,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1073,8 +1198,39 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: true,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
+                            }
+                        }
+                    },
+                );
+            }
+
+            it "accepts --log-dir" {
+                let path = env!("CARGO_MANIFEST_DIR");
+                check_args(
+                    &["shawl", "run", "--log-dir", path, "--", "foo"],
+                    Cli {
+                        sub: Subcommand::Run {
+                            name: s("Shawl"),
+                            cwd: None,
+                            common: CommonOpts {
+                                pass: None,
+                                restart: false,
+                                no_restart: false,
+                                restart_if: vec![],
+                                restart_if_not: vec![],
+                                stop_timeout: None,
+                                no_log: false,
+                                no_log_cmd: false,
+                                log_dir: Some(p(path)),
+                                pass_start_args: false,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1097,8 +1253,122 @@ speculate::speculate! {
                                 stop_timeout: None,
                                 no_log: false,
                                 no_log_cmd: false,
-                                command: vec![s("foo")],
+                                log_dir: None,
                                 pass_start_args: true,
+                                env: vec![],
+                                path: vec![],
+                                command: vec![s("foo")],
+                            }
+                        }
+                    },
+                );
+            }
+
+            it "accepts --env" {
+                check_args(
+                    &["shawl", "add", "--env", "FOO=bar", "--name", "foo", "--", "foo"],
+                    Cli {
+                        sub: Subcommand::Add {
+                            name: s("foo"),
+                            cwd: None,
+                            common: CommonOpts {
+                                pass: None,
+                                restart: false,
+                                no_restart: false,
+                                restart_if: vec![],
+                                restart_if_not: vec![],
+                                stop_timeout: None,
+                                no_log: false,
+                                no_log_cmd: false,
+                                log_dir: None,
+                                pass_start_args: false,
+                                env: vec![(s("FOO"), s("bar"))],
+                                path: vec![],
+                                command: vec![s("foo")],
+                            }
+                        }
+                    },
+                );
+            }
+
+            it "accepts --env multiple times" {
+                check_args(
+                    &["shawl", "add", "--env", "FOO=1", "--env", "BAR=2", "--name", "foo", "--", "foo"],
+                    Cli {
+                        sub: Subcommand::Add {
+                            name: s("foo"),
+                            cwd: None,
+                            common: CommonOpts {
+                                pass: None,
+                                restart: false,
+                                no_restart: false,
+                                restart_if: vec![],
+                                restart_if_not: vec![],
+                                stop_timeout: None,
+                                no_log: false,
+                                no_log_cmd: false,
+                                log_dir: None,
+                                pass_start_args: false,
+                                env: vec![(s("FOO"), s("1")), (s("BAR"), s("2"))],
+                                path: vec![],
+                                command: vec![s("foo")],
+                            }
+                        }
+                    },
+                );
+            }
+
+            it "accepts --path" {
+                let path = env!("CARGO_MANIFEST_DIR");
+                check_args(
+                    &["shawl", "add", "--path", path, "--name", "foo", "--", "foo"],
+                    Cli {
+                        sub: Subcommand::Add {
+                            name: s("foo"),
+                            cwd: None,
+                            common: CommonOpts {
+                                pass: None,
+                                restart: false,
+                                no_restart: false,
+                                restart_if: vec![],
+                                restart_if_not: vec![],
+                                stop_timeout: None,
+                                no_log: false,
+                                no_log_cmd: false,
+                                log_dir: None,
+                                pass_start_args: false,
+                                env: vec![],
+                                path: vec![p(path)],
+                                command: vec![s("foo")],
+                            }
+                        }
+                    },
+                );
+            }
+
+            it "accepts --path multiple times" {
+                let path1 = format!("{}/target", env!("CARGO_MANIFEST_DIR"));
+                let path2 = format!("{}/src", env!("CARGO_MANIFEST_DIR"));
+                check_args(
+                    &["shawl", "add", "--path", &path1, "--path", &path2, "--name", "foo", "--", "foo"],
+                    Cli {
+                        sub: Subcommand::Add {
+                            name: s("foo"),
+                            cwd: None,
+                            common: CommonOpts {
+                                pass: None,
+                                restart: false,
+                                no_restart: false,
+                                restart_if: vec![],
+                                restart_if_not: vec![],
+                                stop_timeout: None,
+                                no_log: false,
+                                no_log_cmd: false,
+                                log_dir: None,
+                                pass_start_args: false,
+                                env: vec![],
+                                path: vec![p(&path1), p(&path2)],
+                                command: vec![s("foo")],
                             }
                         }
                     },
@@ -1155,8 +1425,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl"],
@@ -1177,8 +1450,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "\"C:/Program Files/shawl\""],
@@ -1199,8 +1475,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--restart"],
@@ -1221,8 +1500,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--no-restart"],
@@ -1243,8 +1525,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--restart-if", "0"],
@@ -1265,8 +1550,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--restart-if", "1,10"],
@@ -1287,8 +1575,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--restart-if-not", "0"],
@@ -1309,8 +1600,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--restart-if-not", "1,10"],
@@ -1331,8 +1625,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--pass", "0"],
@@ -1353,8 +1650,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--pass", "1,10"],
@@ -1375,8 +1675,11 @@ speculate::speculate! {
                         stop_timeout: Some(3000),
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--stop-timeout", "3000"],
@@ -1397,8 +1700,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--cwd", "C:/foo"],
@@ -1419,15 +1725,17 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--cwd", "\"C:/Program Files/foo\""],
             );
         }
 
-
         it "handles --no-log" {
             assert_eq!(
                 construct_shawl_run_args(
@@ -1442,8 +1750,11 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: true,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--no-log"],
@@ -1463,14 +1774,67 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: true,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--no-log-cmd"],
             );
         }
 
+        it "handles --log-dir without spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: Some("C:/foo".to_string()),
+                        pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--log-dir", "C:/foo"],
+            );
+        }
+
+        it "handles --log-dir with spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: Some("C:/foo bar/hello".to_string()),
+                        pass_start_args: false,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--log-dir", "\"C:/foo bar/hello\""],
+            );
+        }
+
         it "handles --pass-start-args" {
             assert_eq!(
                 construct_shawl_run_args(
@@ -1485,13 +1849,166 @@ speculate::speculate! {
                         stop_timeout: None,
                         no_log: false,
                         no_log_cmd: false,
-                        command: vec![s("foo")],
+                        log_dir: None,
                         pass_start_args: true,
+                        env: vec![],
+                        path: vec![],
+                        command: vec![s("foo")],
                     }
                 ),
                 vec!["run", "--name", "shawl", "--pass-start-args"],
             );
         }
+
+        it "handles --env without spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![(s("FOO"), s("bar"))],
+                        path: vec![],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--env", "FOO=bar"],
+            );
+        }
+
+        it "handles --env with spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![(s("FOO"), s("bar baz"))],
+                        path: vec![],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--env", "\"FOO=bar baz\""],
+            );
+        }
+
+        it "handles --env multiple times" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![(s("FOO"), s("1")), (s("BAR"), s("2"))],
+                        path: vec![],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--env", "FOO=1", "--env", "BAR=2"],
+            );
+        }
+
+        it "handles --path without spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![],
+                        path: vec![s("C:/foo")],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--path", "C:/foo"],
+            );
+        }
+
+        it "handles --path with spaces" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![],
+                        path: vec![s("C:/foo bar")],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--path", "\"C:/foo bar\""],
+            );
+        }
+
+        it "handles --path multiple times" {
+            assert_eq!(
+                construct_shawl_run_args(
+                    &s("shawl"),
+                    &None,
+                    &CommonOpts {
+                        pass: None,
+                        restart: false,
+                        no_restart: false,
+                        restart_if: vec![],
+                        restart_if_not: vec![],
+                        stop_timeout: None,
+                        no_log: false,
+                        no_log_cmd: false,
+                        log_dir: None,
+                        pass_start_args: false,
+                        env: vec![],
+                        path: vec![s("C:/foo"), s("C:/bar")],
+                        command: vec![s("foo")],
+                    }
+                ),
+                vec!["run", "--name", "shawl", "--path", "C:/foo", "--path", "C:/bar"],
+            );
+        }
     }
 
     describe "prepare_command" {
